/**
 * main.js
 *
 *
 *
 */

const oracledb = require('oracledb');
const fs = require('fs');
const yaml = require('js-yaml');

const COMMON_EXEC_PARAMS = { resultSet : true,
                        outFormat: oracledb.OUT_FORMAT_OBJECT } ;
const ACTOR_USERNAME = "guias_trabajo_error";


// https://stackoverflow.com/questions/951021/what-is-the-javascript-version-of-sleep
// sleep time expects milliseconds
function sleep (time) {
  return new Promise((resolve) => setTimeout(resolve, time));
}

async function delete_guide_if_exists( conn, work_guide_record ){
  await conn.execute(`delete from df_cmn_aum_xx_tsk_job_gud
    where job_gud_idn = :idt_sqn`, 
    {
      idt_sqn : work_guide_record.IDT_SQN
    },
    {
      autoCommit : true 
    });
  return conn.execute(`delete from df_cmn_aum_xx_tsk_job_gud
    where idt_sqn = :idt_sqn`,
  {
    idt_sqn : work_guide_record.IDT_SQN
  },
  {
    autoCommit : true
  });
}


async function insert_into_guide( conn, work_guide_record ){

  return conn.execute(`insert into df_cmn_aum_xx_tsk_job_gud
     (idt_sqn,
      job_gud_nam,
      nod_typ_idn,
      nod_nam,
      job_gud_idn,
      vrs_nbr,
      vrs_dat,
      cod_gud,
      aum_tsk_dsp,
      ord_nbr,
      ins_dat,
      ins_usr_val)
      values
     (:idt_sqn,
       :job_gud_nam,
       :nod_typ_idn,
       :nod_nam,
       :job_gud_idn,
       :vrs_nbr,
       :vrs_dat,
       :cod_gud,
       :aum_tsk_dsp,
       :ord_nbr,
       systimestamp,
       :actor_username)`,
      {
        idt_sqn     : work_guide_record.IDT_SQN,
        job_gud_nam : work_guide_record.JOB_GUD_NAM,
        nod_typ_idn : work_guide_record.NOD_TYP_IDN,
        nod_nam     : work_guide_record.NOD_NAM,
        job_gud_idn : work_guide_record.JOB_GUD_IDN,
        vrs_nbr     : work_guide_record.VRS_NBR,
        vrs_dat     : new Date( Date.parse(work_guide_record.VRS_DAT ) ),
        cod_gud     : work_guide_record.COD_GUD,
        aum_tsk_dsp : work_guide_record.AUM_TSK_DSP,
        ord_nbr     : work_guide_record.ORD_NBR,
        actor_username : ACTOR_USERNAME
      },
      {
        autoCommit : true
      } );
}



async function main() {
	let conn;

	try {
		const config = yaml.load(fs.readFileSync('config.yaml', 'utf8'));

    console.error("Openning db connection....");
    conn = await oracledb.getConnection( config.db );
    conn.autoCommit = true;

    let work_guides_data = JSON.parse( fs.readFileSync( 'guias_trabajo_error.json' ) );
    let dependency_errors = [];
    for( work_guide_record of work_guides_data ){
      console.info( work_guide_record );
      console.info( "ejecutando borrado..." ); 
      await delete_guide_if_exists( conn, work_guide_record );
      console.info( "ejecutando insercion..." );
      await insert_into_guide( conn, work_guide_record );
    }

  } catch (ex) {
	  console.log(ex);
	} finally {
		try{
			console.log("closing db connection...");
			await conn?conn.close():0;
		}catch( ex2 ){
			console.log(ex2);
			// we can ignore this error in this step
		}
	}
}



main();
